<?php
/*
Plugin Name: WooCommerce Direct Free Download
Plugin URI: http://codecanyon.net/item/direct-download-for-woocommerce/6320429
Description: Specially for WooCommerce stores. This plugin hides the button "Add to cart" and creates one to download the downloadable product directly. It only applies for virtual, downloadable and free products.  
Author: Kamalyon.com
Version: 1.16
Author URI: http://kamalyon.com/
*/

/**
 * Check if plugin must flush rules
 */
function dfd_check_flush_rules() {
	//Get current plugin version
	$plugin_data = get_plugin_data(__FILE__);
	$plugin_version = $plugin_data['Version'];
	
	//Get last flush version
	$last_flush_version = get_option('dfd_last_flush_version');
	
	//Flush rules and overwrite the flush version
	if ($plugin_version != $last_flush_version) {

		global $wp_rewrite;
		$wp_rewrite->flush_rules();
		update_option('dfd_last_flush_version', $plugin_version);
	}
}
add_action('admin_init', 'dfd_check_flush_rules');

/**
 * Activate plugin
 */
function dfd_activate() {
	dfd_check_flush_rules();
}
register_activation_hook( __FILE__, 'dfd_activate' );

/**
 * Deactivate plugin
 */
function dfd_deactivate() {
	delete_option('dfd_last_flush_version');
	global $wp_rewrite;
	$wp_rewrite->flush_rules();
}
register_deactivation_hook( __FILE__, 'dfd_deactivate' );

/**
 * Enqueue scripts and styles
 */
function dfd_enqueue_scripts() {
	wp_enqueue_script('jquery');
	wp_enqueue_style('direct-free-download-css', plugins_url( '/css/direct-free-download.css', __FILE__ ));
	wp_register_script('dfd-js', plugins_url('/js/woocommerce-direct-free-download.js', __FILE__ ), 'jquery' );
	wp_enqueue_script('dfd-js');
}
add_action('wp_enqueue_scripts', 'dfd_enqueue_scripts');

//Returns true if the product/variation is free, downloadable and vitual.
function dfd_is_virtual_free_download_product($post) {
	
	if (empty($post)) return false; 
	
	$the_product = get_product($post);

	if (!$the_product) {
		//its not a product or product not found
		return false;
	}
	$is_virtual = $the_product->is_virtual();
	$price = $the_product->get_price();//$the_product->regular_price;
	$sale_price = $the_product->sale_price;
	$is_downloadable = $the_product->is_downloadable();
	
	if ($sale_price == "0" || $sale_price == "0.00"){
		$price = $sale_price;
	}
	if($price == 0 && $is_virtual == 1 && $is_downloadable == 1){
		return true;
	}
	return false;
}

//Add url variables
function dfd_query_vars($qvars) {
	$qvars[] = 'dfd_action';
	$qvars[] = 'file_id';
	return $qvars;
}
add_filter('query_vars', 'dfd_query_vars');

//Load the page specified by url
function dfd_load_template() {
	global $wp_query;
	if (isset($wp_query->query_vars['dfd_action'])) {
	    if ($wp_query->query_vars['dfd_action'] == 'download'){
	        include('direct-download.php');
	        exit;
	    }
	}
}
add_filter('template_redirect', 'dfd_load_template');
 
//Add new rules permalink
function dfd_generate_rewrite_rules($wp_rewrite) {
    $new_rules = array(
        '^direct-download/([^/]*)/?' => 'index.php?dfd_action=download&file_id=$matches[1]'
    );
    $wp_rewrite->rules = array_merge($new_rules, $wp_rewrite->rules);
}
add_action('generate_rewrite_rules', 'dfd_generate_rewrite_rules', 1);


//Display customize button after the product info (Simple product)
function dfd_display_direct_free_download_button() {
	global $product, $woocommerce;
	
	$download_button_text = get_option('download_button_text');
	$download_intro_text = get_option('download_intro_text');
	
	if (empty($product)) return;
	
	//In woocommerce 2.1 the products files has changed its structure
	if (version_compare( $woocommerce->version, '2.1', '<' )) {
		//compatibility for old woocommerce versions (old code)
		if (dfd_is_virtual_free_download_product($product)) {
			$link_product = get_post_meta($product->id, '_file_paths',true);
			if(!empty($link_product)){
				?>
				<?php if (!empty($download_intro_text)) { echo __($download_intro_text, 'directfreedownload'); }?>
				<?php 
				foreach ($link_product as $link){
					$path_parts = pathinfo($link);
					$basename_file = $path_parts['basename'];
					$link_download = $product->id.'|'.$link; ?>
					<div class="dfd-download-buttons">
						<h3><?php echo $basename_file; ?></h3>
						<?php if(get_option('woocommerce_downloads_require_login') == 'yes'){
							//If the id is 0, the user is not identified
							if(!is_user_logged_in()){ ?>
								<a href="<?php echo get_permalink( get_option('woocommerce_myaccount_page_id') ); ?>" class="single_add_to_cart_button button alt direct-free-download-button"><?php _e('Login for download','directfreedownload'); ?></a>
							<?php }else{ ?>
								<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
							<?php } ?>
						<?php }else{ ?>
							<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
						<?php } ?>
						<?php 
						//Display download counter
						/*$count = dfd_get_download_count($product->id); 
						$download_count = $count.' '.__('downloads', 'directfreedownload');
						if ($count == 1) { $download_count = $count.' '.__('download', 'directfreedownload'); }?>
						<span class="dfd-download-count"><?php echo $download_count;?></span>
						*/?>
					</div>
					<?php 
				}
			}
		}
	
	}else{
		//Code for woocommerce versions higher than 2.1 (new code)
		if (dfd_is_virtual_free_download_product($product)) {
			$product_files = $product->get_files();

			if(!empty($product_files)){
				?>
				<?php if (!empty($download_intro_text)) { echo __($download_intro_text, 'directfreedownload'); }?>
				<?php 
				foreach ($product_files as $product_file) { 
					//checks if the file is full configured
					if (!isset($product_file['file']) or empty($product_file['file'])) {
						continue;
					}
					$link = do_shortcode($product_file['file']);
					
					//gets the filename
					if (isset($product_file['name']) and !empty($product_file['name'])) {
						$filename =	$product_file['name'];
					}else{
						$path_parts = pathinfo($link);
						$filename = $path_parts['basename'];
					}
					
					$link_download = $product->id.'|'.$link; ?>
					<div class="dfd-download-buttons">
						<h3><?php echo $filename; ?></h3>
						<?php if(get_option('woocommerce_downloads_require_login') == 'yes'){
							//If the id is 0, the user is not identified
							if(!is_user_logged_in()){ ?>
								<a href="<?php echo get_permalink( get_option('woocommerce_myaccount_page_id') ); ?>" class="single_add_to_cart_button button alt direct-free-download-button"><?php _e('Login for download','directfreedownload'); ?></a>
							<?php }else{ ?>
								<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
							<?php } ?>
						<?php }else{ ?>
							<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
						<?php } ?>
						<?php 
						//Display download counter
						/*$count = dfd_get_download_count($product->id); 
						$download_count = $count.' '.__('downloads', 'directfreedownload');
						if ($count == 1) { $download_count = $count.' '.__('download', 'directfreedownload'); }?>
						<span class="dfd-download-count"><?php echo $download_count;?></span>
						*/?>
					</div>
				<?php }
			}
		}
	}
	
}
add_action( 'woocommerce_simple_add_to_cart', 'dfd_display_direct_free_download_button', 60 );

//Display download button on product lists
function dfd_display_button_in_product_list(){
	global $product, $woocommerce;
	
	$download_button_text = get_option('download_button_text');
	
	if (empty($product)) return;
	
	//In woocommerce 2.1 the products files has changed its structure
	if (version_compare( $woocommerce->version, '2.1', '<' )) {
		//compatibility for old woocommerce versions (old code)
		if (dfd_is_virtual_free_download_product($product)) {
			$link_product = get_post_meta($product->id, '_file_paths',true);
			if(!empty($link_product)){
				foreach ($link_product as $link){
					$path_parts = pathinfo($link);
					$basename_file = $path_parts['basename'];
					$link_download = $product->id.'|'.$link; ?>
					<div class="dfd-download-buttons-on-list">
						<?php if(get_option('woocommerce_downloads_require_login') == 'yes'){
							//If the id is 0, the user is not identified
							if(!is_user_logged_in()){ ?>
								<a href="<?php echo get_permalink( get_option('woocommerce_myaccount_page_id') ); ?>" class="single_add_to_cart_button button alt direct-free-download-button"><?php _e('Login for download','directfreedownload'); ?></a>
							<?php }else{ ?>
								<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
							<?php } ?>
						<?php }else{ ?>
							<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
						<?php } ?>
					</div>
					<?php 
				}
			}
		}
	
	}else{
		//Code for woocommerce versions higher than 2.1 (new code)
		if (dfd_is_virtual_free_download_product($product)) {
			$product_files = $product->get_files();

			if(!empty($product_files)){
				foreach ($product_files as $product_file) { 
					//checks if the file is full configured
					if (!isset($product_file['file']) or empty($product_file['file'])) {
						continue;
					}
					$link = $product_file['file'];
					
					//gets the filename
					if (isset($product_file['name']) and !empty($product_file['name'])) {
						$filename =	$product_file['name'];
					}else{
						$path_parts = pathinfo($link);
						$filename = $path_parts['basename'];
					}
					
					$link_download = dfd_get_download_url($product->id, $link);
					 ?>
					<div class="dfd-download-buttons-on-list">
						<?php if(get_option('woocommerce_downloads_require_login') == 'yes'){
							//If the id is 0, the user is not identified
							if(!is_user_logged_in()){ ?>
								<a href="<?php echo get_permalink( get_option('woocommerce_myaccount_page_id') ); ?>" class="single_add_to_cart_button button alt direct-free-download-button"><?php _e('Login for download','directfreedownload'); ?></a>
							<?php }else{ ?>
								<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
							<?php } ?>
						<?php }else{ ?>
							<a href="<?php bloginfo('url'); ?>/direct-download/<?php echo base64_encode($link_download); ?>/" class="single_add_to_cart_button button alt direct-free-download-button"><?php echo $download_button_text; ?></a>
						<?php } ?>
					</div>
				<?php }
			}
		}
	}
}

$show_download_button_on_list = get_option('download_button_on_list');
if($show_download_button_on_list == "checked"){
	add_action('woocommerce_after_shop_loop_item', 'dfd_display_button_in_product_list', 60);
}

//Get download url
function dfd_get_download_url($product_id ,$link) {
	$link_download = $product_id.'|'.$link;
	$path_parts = pathinfo($link);

	//If link download is dropbox url, add parameter dl=1
	if (strpos($path_parts['dirname'],'dropbox') !== false) {
		$link_download = $product_id.'|'.$link.'?dl=1';
	}
	return $link_download;
}


//Display customize button after the product info (Variable product)
function dfd_display_direct_free_download_button_variable() { 
	$download_button_text = get_option('download_button_text'); 
	$download_intro_text = get_option('download_intro_text');
	?>
	<div id="dfd_variations_button_container"  style="display:none">
		<?php if (!empty($download_intro_text)) { echo __($download_intro_text, 'directfreedownload'); }?>
		<div id="dfd_variations_button" class="dfd-download-buttons" data-product-id="<?php echo $product->id;?>">
			<h3></h3>
			<?php if(get_option('woocommerce_downloads_require_login') == 'yes'){
				//If the id is 0, the user is not identified
				if(!is_user_logged_in()){ ?>
					<a href="<?php echo get_permalink( get_option('woocommerce_myaccount_page_id') ); ?>" class="single_add_to_cart_button button alt direct-free-download-button"><?php _e('Login for download','directfreedownload'); ?></a>
				<?php }else{ ?>
					<a href="" id="dfd_download_button" class="single_add_to_cart_button button alt direct-free-download-button dfd_available_button"><?php echo $download_button_text; ?></a>
				<?php }
				}else{ ?>
					<a href="" id="dfd_download_button" class="single_add_to_cart_button button alt direct-free-download-button dfd_available_button"><?php echo $download_button_text; ?></a>
			<?php } ?>
		</div>
	</div>
<?php 
}
add_action( 'woocommerce_variable_add_to_cart', 'dfd_display_direct_free_download_button_variable', 60 );

//Hide add to cart button
function dfd_hide_add_to_cart($purchasable){
	global $product;
	
	if (empty($product)) return $purchasable;
	
	if (dfd_is_virtual_free_download_product($product)) {
		$purchasable = false;
	}else{
		$purchasable = true;
	}
	return $purchasable;
}
add_filter( 'woocommerce_is_purchasable', 'dfd_hide_add_to_cart', 10, 2 );

//Prepare the data for the download button in case of a variation.
function dfd_woocommerce_available_variation($variation_data,$product,$variation){
	//Assign a new parameter in the variation_data array which contain a true or false depending on the variation is free, downloadable and virtual.
	$variation_data['dfd_available_variation'] = dfd_is_virtual_free_download_product($variation->get_variation_id());
	
	//If so, retrieve the file_paths and encrypt the url.
	if ($variation_data['dfd_available_variation']) {
		
		$links = array();
		$name_files_array = array();
		
		global $woocommerce;
		//In woocommerce 2.1 the products files has changed its structure
		if (version_compare($woocommerce->version, '2.1', '<' )) {
			//compatibility for old woocommerce versions (old code)
			$link_product = get_post_meta($variation->get_variation_id(), '_file_paths',true);
			
			if(!empty($link_product)){
				foreach ($link_product as $link){
					$path_parts = pathinfo($link);
					$basename_file = $path_parts['basename'];
					
					$link_download = $variation->get_variation_id().'|'.$link; 
					$link_download = get_bloginfo('url') . "/direct-download/" . base64_encode($link_download) . "/";
					$links[] = $link_download;
					$name_files_array[] = $basename_file;
				}
			}
			
		}else{
			//Code for woocommerce versions higher than 2.1 (new code)
			$product_files = $variation->get_files();
			if(!empty($product_files)){
				foreach ($product_files as $product_file) {
					//checks if the file is full configured
					if (!isset($product_file['file']) or empty($product_file['file'])) {
						continue;
					}
					$link = $product_file['file'];
					
					//gets the filename
					if (isset($product_file['name']) and !empty($product_file['name'])) {
						$filename =	$product_file['name'];
					}else{
						$path_parts = pathinfo($link);
						$filename = $path_parts['basename'];
					}
					
					$link_download = $variation->get_variation_id().'|'.$link; 
					$link_download = get_bloginfo('url') . "/direct-download/" . base64_encode($link_download) . "/";
					$links[] = $link_download;
					$name_files_array[] = $filename;
				}
			}
		}
		
		$variation_data['dfd_name_files'] = $name_files_array;
		$variation_data['dfd_available_urls'] = $links;
			
		//Use this new position in the array to know how many links containing file_paths.
		$variation_data['dfd_available_urls_count'] = count($links);
		
	}else{
		$variation_data['dfd_available_urls'] = "";
		$variation_data['dfd_available_urls_count'] = 0;
	}
	
	return $variation_data;
}
add_filter( 'woocommerce_available_variation', 'dfd_woocommerce_available_variation', 10, 3 );

//Add configure text button
add_action( 'admin_menu', 'direct_download_admin_menu',99);
function direct_download_admin_menu() {
	add_submenu_page(
		'woocommerce', 
		'Direct Download',     // page title
		'Direct Download',     // menu title
		'manage_options', 		 // capability
		'direct-download-terms',     // menu slug
		'direct_download_admin_callback' // callback function
	); 
}

//Admin page
function direct_download_admin_callback() {
	global $title;
	
	$download_button_text = $_POST['download_button_text'];
	if (isset($download_button_text)) {
		update_option('download_button_text', $download_button_text);
	}else{
		$download_button_text = get_option('download_button_text');
	}
	
	$download_intro_text = $_POST['download_intro_text'];
	if (isset($download_intro_text)) {
		update_option('download_intro_text', $download_intro_text);
	}else{
		$download_intro_text = get_option('download_intro_text');
	}
	
	$show_download_button_on_list = $_POST['download_button_on_list'];
	if( $show_download_button_on_list == true ){
		$show_download_button_on_list = "checked";
		update_option('download_button_on_list', $show_download_button_on_list);
	}else{
		$show_download_button_on_list = "no_checked";
		update_option('download_button_on_list', $show_download_button_on_list);
	} ?>
	
	<div class="wrap">
		<h1><?php echo $title?></h1>
		<form action="" method="post">
			<table class="form-table">
				<tbody>
					<tr valign="top">
						<th>
							<label for="download_button_text"><?php _e('Download button text','directfreedownload'); ?></label>
						</th>
						<td class="titledesc">
							<input name="download_button_text" id="download_button_text" type="text" value="<?php echo $download_button_text;?>" class="" size="20px" />
						</td>
					</tr>
					<tr valign="top">
						<th>
							<label for="download_intro_text"><?php _e('Introduction text','directfreedownload'); ?></label>
						</th>
						<td class="titledesc">
							
							<textarea name="download_intro_text" id="download_intro_text" cols="40" rows="4"><?php echo $download_intro_text;?></textarea>
							<p class="description"><?php _e('The content of this textarea will be shown before the download buttons.','directfreedownload'); ?> <?php _e('You can add HTML if you want.','directfreedownload'); ?></p>
						</td>
					</tr>					
					<tr>
						<th>
							<label for="download_button_on_list"><?php _e('Show download button on product lists','directfreedownload'); ?></label>
						</th>
						<td class="titledesc">
							<input type="checkbox" name="download_button_on_list" id="download_button_on_list" <?php echo $show_download_button_on_list; ?> />
						</td>	
		         	</tr>
	             </tbody>
	        </table>
	        <br/>
	        <input type="submit" class="button-primary" value="Save changes"/>
        </form>
	</div>
	<?php 
}


//Adds a metabox to the product page to display the download counter
function dfd_add_meta_box() {
	$screens = array('product');
	foreach ($screens as $screen) {
		add_meta_box(
		'dfd_directfreedownload',
		__( 'Direct Download', 'directfreedownload'),
		'dfd_meta_box_callback',
		$screen,
		'side'
		);
	}
}
add_action('add_meta_boxes', 'dfd_add_meta_box');

//Displays the metabox
function dfd_meta_box_callback($post) {
	$count = dfd_get_download_count($post->ID);
	if ($count == 1) {
		echo $count.' '.__('direct download', 'directfreedownload');
	}else{
		echo $count.' '.__('direct downloads', 'directfreedownload');
	}
}

//Gets the download count for the product_id
function dfd_get_download_count($post_id) {
	//Product count
	$count = get_post_meta($post_id, '_dfd_download_count', true);
	if (empty($count)) { $count = 0; }
	
	//Check if variations has downloads too.
	$product = wc_get_product($post_id); 
	if (!empty($product) && method_exists($product,'get_available_variations')) {
		$available_variations = $product->get_available_variations();
		foreach($available_variations as $prod_variation) {
			$post_id = $prod_variation['variation_id'];
			$variation_count = get_post_meta($post_id, '_dfd_download_count', true);
			if (empty($variation_count)) { $variation_count = 0; }
			$count = $count + $variation_count;
		}
	}
	return $count;
}

//Gets the total downloads of all products
function dfd_get_all_downloads_count() {
	$products = get_posts(array('post_type' => 'product', 'posts_per_page'   => -1));
	$total_downloads = 0;
	foreach ($products as $product) {
		$count = get_post_meta($product->ID, '_dfd_download_count', true);
		if (empty($count)) { $count = 0; }
		$product = wc_get_product($product->ID);
		if (!empty($product) && method_exists($product,'get_available_variations')) {
			$available_variations = $product->get_available_variations();
			foreach($available_variations as $prod_variation) {
				$post_id = $prod_variation['variation_id'];
				$variation_count = get_post_meta($post_id, '_dfd_download_count', true);
				if (empty($variation_count)) { $variation_count = 0; }
				$count = $count + $variation_count;
			}
		}
		//$count = get_post_meta($product->ID, '_dfd_download_count', true);
		if (!empty($count)) { $total_downloads+= $count; }
	}
	return $total_downloads;
}

add_filter('upload_mimes','add_custom_mime_types');
function add_custom_mime_types($mimes){
	return array_merge($mimes,array (
		// Image formats
		'jpg|jpeg|jpe'		=> 'image/jpeg',
		'gif'				=> 'image/gif',
		'png'				=> 'image/png',
		'bmp'				=> 'image/bmp',
		'tif|tiff'			=> 'image/tiff',
		'ico'				=> 'image/x-icon',
		
		// Video formats
		'asf|asx'			=> 'video/x-ms-asf',
		'wmv'				=> 'video/x-ms-wmv',
		'wmx'				=> 'video/x-ms-wmx',
		'wm'				=> 'video/x-ms-wm',
		'avi'				=> 'video/avi',
		'divx'				=> 'video/divx',
		'flv'				=> 'video/x-flv',
		'mov|qt'			=> 'video/quicktime',
		'mpeg|mpg|mpe'		=> 'video/mpeg',
		'mp4|m4v'			=> 'video/mp4',
		'ogv'				=> 'video/ogg',
		'webm'				=> 'video/webm',
		'mkv'				=> 'video/x-matroska',
			
		// Text formats
		'txt|asc|c|cc|h'	=> 'text/plain',
		'csv'				=> 'text/csv',
		'tsv'				=> 'text/tab-separated-values',
		'ics'				=> 'text/calendar',
		'rtx'				=> 'text/richtext',
		'css'				=> 'text/css',
		'htm|html'			=> 'text/html',
		
		// Audio formats
		'mp3|m4a|m4b'		=> 'audio/mpeg',
		'ra|ram'			=> 'audio/x-realaudio',
		'wav'				=> 'audio/wav',
		'ogg|oga'			=> 'audio/ogg',
		'mid|midi'			=> 'audio/midi',
		'wma'				=> 'audio/x-ms-wma',
		'wax'				=> 'audio/x-ms-wax',
		'mka'				=> 'audio/x-matroska',
		
		// Misc application formats
		'rtf'				=> 'application/rtf',
		'js'				=> 'application/javascript',
		'pdf'				=> 'application/pdf',
		'swf'				=> 'application/x-shockwave-flash',
		'class'				=> 'application/java',
		'tar'				=> 'application/x-tar',
		'zip'				=> 'application/zip',
		'gz|gzip'			=> 'application/x-gzip',
		'rar'				=> 'application/rar',
		'7z'				=> 'application/x-7z-compressed',
		'exe'				=> 'application/x-msdownload'
	));
}

// Function that outputs the contents of the dashboard widget
function dfd_dashboard_widget( $post, $callback_args ) {
	echo dfd_get_all_downloads_count()." total direct downloads ";
}

// Function used in the action hook
function dfd_add_dashboard_widgets() {
	wp_add_dashboard_widget('dashboard_widget', 'WooCommerce Direct Download', 'dfd_dashboard_widget');
}

// Register the new dashboard widget with the 'wp_dashboard_setup' action
add_action('wp_dashboard_setup', 'dfd_add_dashboard_widgets' );
?>